/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.GCLCConstants.java
 * Created on: Jun 8, 2016
 */
package fr.bigeon.gclc;

import java.util.ArrayList;
import java.util.List;

import fr.bigeon.gclc.exception.CommandParsingException;

/** A Utility class for GCLC
 * <p>
 * This class offers a method to split a line or arguments into the list of
 * arguments.
 *
 * @author Emmanuel Bigeon */
public final class GCLCConstants {

    /** The escaping character. */
    private static final char ESCAPING_CHAR = getSystemEscapingChar();

    /** Hide utility class constructor. */
    private GCLCConstants() {
        // utility class
    }

    /** Get the end of a string argument.
     *
     * @param cmd the command to parse
     * @param startIndex the starting point of the parsing
     * @param index the index of the current position
     * @return the argument
     * @throws CommandParsingException if the end of string does not mark end of
     *             command and is not followed by a space */
    private static String endOfString(final String cmd, final int startIndex,
                                      final int index) throws CommandParsingException {
        if (index < cmd.length() && cmd.charAt(index) != ' ') {
            throw new CommandParsingException("Misplaced quote"); //$NON-NLS-1$
        }
        return cmd.substring(startIndex + 1, index - 1);
    }

    /** Get the excaping character.
     *
     * @return the escaping character */
    private static char getSystemEscapingChar() {
        return '\\';
    }

    /** Remove escaping characters from the string.
     *
     * @param arg the string to remove excaping character from
     * @return the string without escape character */
    private static String removeEscaped(final String arg) {
        final StringBuilder builder = new StringBuilder();
        int index = 0;
        int endIndex = arg.indexOf(ESCAPING_CHAR);
        while (endIndex != -1) {
            builder.append(arg.subSequence(index, endIndex));
            index = endIndex + 1;
            endIndex = arg.indexOf(ESCAPING_CHAR, index + 1);
        }
        builder.append(arg.substring(index));
        return builder.toString();
    }

    /** Splits a command in the diferrent arguments.
     *
     * @param cmd the command to split in its parts
     * @return the list of argument preceded by the command name
     * @throws CommandParsingException if the parsing of the command failed */
    public static List<String> splitCommand(final String cmd) throws CommandParsingException {
        final List<String> args = new ArrayList<>();
        // parse the string to separate arguments
        int index = 0;
        int startIndex = 0;
        boolean escaped = false;
        boolean inString = false;
        while (index < cmd.length()) {
            final char c = cmd.charAt(index);
            index++;
            if (escaped || c == ESCAPING_CHAR) {
                escaped = !escaped;
                continue;
            }
            if (c == ' ' && !inString) {
                final String arg = cmd.substring(startIndex, index - 1);
                if (!arg.isEmpty()) {
                    args.add(removeEscaped(arg));
                }
                startIndex = index;
            } else if (c == '"') {
                if (inString) {
                    args.add(endOfString(cmd, startIndex, index));
                    index++;
                    startIndex = index;
                }
                inString = startIndex == index - 1;
            }
        }
        if (startIndex < cmd.length()) {
            final String arg = cmd.substring(startIndex, cmd.length());
            args.add(arg);
        }
        return args;
    }

}
